/********** Localsolver_faces_Shields.lsp **********/


use io;

/* Reads instance data. */
function input() 
{
    local usage = "Usage: Localsolver_faces_Shields.lsp "
        + "inFileName=inputFile [solFileName=outputFile] [lsTimeLimit=timeLimit]";

    if (inFileName == nil) throw usage;

    local inFile = io.openRead(inFileName);
    n1 = inFile.readInt();
    n2 = inFile.readInt();
    nt = n1+n2;
    ss = inFile.readInt();
    Aux = 0;     
    for [l in 0..ss-1]
    {
        m[l] = inFile.readInt();                
        Aux = Aux + m[l];
    }
    krtmax = Aux;
    local coordx;
    local coordy;
    for [i in 0..n1-1]
    {
        coordx[i] = inFile.readInt();
        coordy[i] = inFile.readInt();
        q1[i] = inFile.readInt();
        q2[i] = 0;
    }
    for [i in n1..nt-1]
    {
        coordx[i] = inFile.readInt();
        coordy[i] = inFile.readInt();
        q2[i] = inFile.readInt();
        q1[i] = 0;
    }
    for [l in 0..ss-1]
    {
        coordx[nt+l] = inFile.readInt();
        coordy[nt+l] = inFile.readInt();
    }
    for [l in 0..ss-1]
    {
        coordx[nt+ss+l] = inFile.readInt();
        coordy[nt+ss+l] = inFile.readInt();
    }
    local dist;
    for [i in 0..nt+2*ss-1]
    {
        for [j in 0..nt+2*ss-1]
        {
            dist = inFile.readInt();
            distMat[i][j] = dist/1000;
        }           
    }
    for [i in 0..nt+2*ss-1]
    {
        for [j in 0..nt+2*ss-1]
        {
            tpoMat[i][j] = inFile.readDouble();
        }           
    }
    BigT = 1000;
    for [i in n1..nt-1]
    {
        for [j in 0..n1-1]
        {
            tpoMat[i][j] = BigT;
        }           
    }
    k = 0;
    for [l in 0..ss-1]
    {
        for [r in 0..m[l]-1]
        {
            k = k+1;
            for [i in 0..nt-1]
            {
                dstini[k][i] = distMat[nt+l][i];
                dstfin[k][i] = distMat[i][nt+ss+l];
                tpoini[k][i] = tpoMat[nt+l][i];
                tpofin[k][i] = tpoMat[i][nt+ss+l];
            }            
        }
    }
}


/* Declares the optimization model. */
function model()
{
    // Sequence of customers visited by each truck.
    customersSequences[k in 1..krtmax] <- list(nt);

    // All customers must be visited by the trucks
    constraint partition[k in 1..krtmax](customersSequences[k]);


    for [k in 1..krtmax]
    {
        local sequence <- customersSequences[k];
        local c <- count(sequence);
        
        
        //Collection points before unloading points       
        routeQuantity1 <- sum(0..c-1, i => q1[sequence[i]]);
        routeQuantity2 <- sum(0..c-1, i => q2[sequence[i]]);
        constraint routeQuantity1 >= routeQuantity2;


        // Distance travelled by truck k
        routeDistances[k] <- sum(1..c-1, i => distMat[sequence[i - 1]][sequence[i]]) + (c > 0 ? (dstini[k][sequence[0]] + dstfin[k][sequence[c - 1]]) : 0);


        // Time travelled by truck k
        routeTimes[k] <- sum(1..c-1, i => tpoMat[sequence[i - 1]][sequence[i]]) + (c > 0 ? (tpoini[k][sequence[0]] + tpoini[k][sequence[c - 1]]) : 0);
    }

    //Maximum Time
    MaxTime <- max[k in 1..krtmax](routeTimes[k]);

    // Total distance travelled
    totalDistance <- sum[k in 1..krtmax](routeDistances[k]);

    // Objective: minimize the Maximum Time first and then the total distance
    minimize MaxTime;
    minimize totalDistance;
}


/* Writes the solution in a file */
function output()
{
    if(solFileName == nil) return;
    local solFile = io.openAppend(solFileName);
    solFile.println(MaxTiempo.value + "  " + totalDistance.value);
}